/*  aqotdd - Afghan's Quote of the day server - An RFC865 compliant server.
    fortune-tcp,fortune-udp,udpbench,tcpbench - RFC865 compliant clients.
    Copyright (C) 2000 Afghan Hound Software

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/*  common.c - common routines to clients and server  */

#include "main.h"

void *xmalloc (size_t size)
{
  register void *value = malloc(size);
  if (value == NULL)
    {
      printf("Help!\n");
      exit(1);
    }
  return value;
}

void *xrealloc(void *original,size_t new_size)
{
  original = realloc(original,new_size);
  if (original == NULL)
    {
     printf("Really Help!\n");
      exit(1);
    }
  return original;
}

char *get_host(char *buf)
{
  char *buffer;
  if (command_host==NULL)
    {
      if ((buffer=getenv(ENV_VARHOST))==NULL)
	{
	  buffer = (char *)xmalloc((strlen(QOTD_DEFHOST)+1)*sizeof(char));
	  strcpy(buffer,QOTD_DEFHOST);
	}
    }
  else
    {
      buffer=command_host;
    }
  return buffer;
}

int get_port(char *proto)
{
  char *buffer;
  int   i;
  if (command_port==NULL)
    {
      if ((buffer=getenv(ENV_VARPORT))==NULL)
	{
	  buffer = (char *)xmalloc((strlen(QOTD_PORTDEF)+1)*sizeof(char));
	  strcpy(buffer,QOTD_PORTDEF);
	}
    }
  else
    {
      buffer=command_port;
    }
  i = htons(atoport(buffer,proto,QOTD_DEFPORT));
  return i;
}

/* From unix-socket FAQ */
int atoport(char *service,char *proto,int def)
{
  long int        lport;
  struct servent *serv;
  char           *errpos;
  /* First try to read it from /etc/services */
	
  serv = getservbyname(service, proto);
  if (serv != NULL)
    {
      lport = ntohs(serv->s_port);
    }
  else 
    { /* Not in services, maybe a number? */
      lport = strtol(service,&errpos,0);
      if ( (errpos[0] != 0) || (lport < 1) || (lport > 65535) )
	{
	  return def;
	}
    }
  return lport;
}

/* Converts ascii text to in_addr struct.  NULL is returned if the address
   can not be found. */
struct in_addr *atoaddr(char *address)
{
  struct hostent *host;
  host = gethostbyname(address);
  if (host != NULL) 
    {
      return (struct in_addr *) *host->h_addr_list;
    }
  return NULL;
}

int parse_client_option(int c,char **v,char *which)
{
  char       next_option;
  extern int opterr; /*Ignore other options silently*/
  int        arg_len,login_flag;
  opterr     = 0;
  login_flag = 0;
  wait_flag  = 0;
  while ((next_option= getopt (c, v, "vhlws:p:"))!= -1)
    switch(next_option)
      {
      case 'v':
        printf("Afghan fortune.%s version %s(%s). Copyright (C) 2000 Afghan Hound.\n\n",which,VERSION,NAME);
	printf("  *** This program comes with ABSOLUTELY NO WARRANTY. *** \n\n");
	printf("You may redistribute copies of Afghan Fortune.%s under the terms\n",which);
	printf("of the GNU General Public License. For more information about\n");
	printf("these matters, see the file named COPYING.\n");
	exit(0);
        break;
      case 'h': 
	printf("Afghan fortune.%s version %s(%s). Copyright (C) 2000 Afghan Hound.\n",which,VERSION,NAME);
	printf("Usage:\n");
	printf("     -s hostname : specify QOTD server hostname.\n");
	printf("     -p port     : specify QOTD server port number.\n");
	printf("     -l          : Add extra blank lines before and after quote.\n");
	printf("     -w          : wait for 0.05761 seconds per character before exiting.\n");
	printf("     -h          : print this help.\n");	
	printf("     -v          : print version infomation.\n");
	exit(0);
	break;
      case 's':
	arg_len=strlen(optarg)+1; /* include \0 */
        command_host=(char *)xmalloc(arg_len*sizeof(char));
        strncpy(command_host, optarg,(arg_len)); 
	break;
      case 'l':
	login_flag = 1;
	break;
      case 'w':
	wait_flag  = 1;
	break;
      case 'p':
	arg_len=strlen(optarg)+1; /* include \0 */
        command_port=(char *)xmalloc(arg_len*sizeof(char));
        strncpy(command_port, optarg,(arg_len));
	break;
      }
  return login_flag;
}

void parse_bench_option(int c,char **v,char *which)
{
  char       next_option;
  extern int opterr; /*Ignore other options silently*/
  int        arg_len,temp;
  opterr        = 0;
  temp          = 0;
  log_con       = 0;
  interval_time = 1;

  while ((next_option= getopt (c, v, "vohr:s:p:i:"))!= -1)
    switch(next_option)
      {
      case 'v':
        printf("Afghan BenchMark.%s version %s(%s). Copyright (C) 2000 Afghan Hound.\n\n",which,VERSION,NAME);
	printf("  *** This program comes with ABSOLUTELY NO WARRANTY. *** \n\n");
	printf("You may redistribute copies of Afghan Fortune.%s under the terms\n",which);
	printf("of the GNU General Public License. For more information about\n");
	printf("these matters, see the file named COPYING.\n");
	exit(0);
        break;
      case 'h': 
	printf("Afghan BenchMark.%s version %s(%s). Copyright (C) 2000 Afghan Hound.\n",which,VERSION,NAME);
	printf("Usage:\n");
	printf("     -s hostname : specify QOTD server hostname.\n");
	printf("     -p port     : specify QOTD server port number.\n");
	printf("     -i time     : seconds between requests.\n");
	printf("     -r time     : seconds between reports.\n");
	printf("     -o          : output responses. (else response ignored).\n");
	printf("     -h          : print this help.\n");	
	printf("     -v          : print version infomation.\n");
	exit(0);
	break;
      case 's':
	arg_len=strlen(optarg)+1; /* include \0 */
        command_host=(char *)xmalloc(arg_len*sizeof(char));
        strncpy(command_host, optarg,(arg_len)); 
	break;
      case 'p':
	arg_len=strlen(optarg)+1; /* include \0 */
	command_port=(char *)xmalloc(arg_len*sizeof(char));
        strncpy(command_port, optarg,(arg_len));
	break;
      case 'r':
	temp = atoi(optarg);
	if ((temp>0)&&(temp<65535))
	  {
	    mark_time=temp;
	  }
	break;
      case 'i':
	temp = atoi(optarg);
	if ((temp>=0)&&(temp<65535))
	  {
	    interval_time=temp;
	  }
	break;
      case 'o':
	log_con = 1;
	break;
      }
}

void check_bench_options()
{
  if ((interval_time)&&(mark_time))
    {
      printf("Warning: -r parameter ignored with non-zero -i.\n");
    }
  
  if ((!(interval_time)&&(!(mark_time))))
    {
      printf("No log time parameter given with 0 -i option. Exiting.\n");
      exit(1);
    }
  if ((!(interval_time))&&(getuid()))
    {
      printf("Permission denied. You must be root to set \"-i\" to 0.\n");
      exit(1);
    }
}
